@@ -33,7 +33,6 @@ from ryu.lib import hub
 from faucet.config_parser import get_config_for_api
 from faucet.valve_ryuapp import EventReconfigure, RyuAppBase
 from faucet.valve_util import dpid_log, kill_on_exception
-from faucet import faucet_experimental_api
 from faucet import faucet_event
 from faucet import faucet_bgp
 from faucet import faucet_dot1x
@@ -42,8 +41,7 @@ from faucet import faucet_metrics
 from faucet import valve_of
 
 
-class EventFaucetExperimentalAPIRegistered(event.EventBase):  # pylint: disable=too-few-public-methods
-    """Event used to notify that the API is registered with Faucet."""
+EXPORT_RYU_CONFIGS = ['echo_request_interval', 'maximum_unreplied_echo_requests']
 
 
 class EventFaucetMaintainStackRoot(event.EventBase):  # pylint: disable=too-few-public-methods
@@ -77,6 +75,7 @@ class EventFaucetFastAdvertise(event.EventBase):  # pylint: disable=too-few-publ
 class EventFaucetEventSockHeartbeat(event.EventBase):  # pylint: disable=too-few-public-methods
     """Event used to trigger periodic events on event sock, causing it to raise an exception if conn is broken."""
 
+
 class Faucet(RyuAppBase):
     """A RyuApp that implements an L2/L3 learning VLAN switch.
 
@@ -85,9 +84,7 @@ class Faucet(RyuAppBase):
     """
     _CONTEXTS = {
         'dpset': dpset.DPSet,
-        'faucet_experimental_api': faucet_experimental_api.FaucetExperimentalAPI,
         }
-    _EVENTS = [EventFaucetExperimentalAPIRegistered]
     _VALVE_SERVICES = {
         EventFaucetMetricUpdate: (None, 5),
         EventFaucetMaintainStackRoot: (None, valves_manager.STACK_ROOT_STATE_UPDATE_TIME),
@@ -106,7 +103,6 @@ class Faucet(RyuAppBase):
 
     def __init__(self, *args, **kwargs):
         super(Faucet, self).__init__(*args, **kwargs)
-        self.api = kwargs['faucet_experimental_api']
         self.prom_client = faucet_metrics.FaucetMetrics(reg=self._reg)
         self.bgp = faucet_bgp.FaucetBgp(
             self.logger, self.exc_logname, self.prom_client, self._send_flow_msgs)
@@ -126,6 +122,12 @@ class Faucet(RyuAppBase):
     def _check_thread_exception(self):
         super(Faucet, self)._check_thread_exception()
 
+    def _export_ryu_config(self):
+        for opt_name in EXPORT_RYU_CONFIGS:
+            value = int(getattr(self.CONF, opt_name))
+            config_labels = dict(param=opt_name)
+            self.prom_client.ryu_config.labels(**config_labels).set(value)
+
     @kill_on_exception(exc_logname)
     def start(self):
         super(Faucet, self).start()
@@ -134,6 +136,7 @@ class Faucet(RyuAppBase):
         prom_port = int(self.get_setting('PROMETHEUS_PORT'))
         prom_addr = self.get_setting('PROMETHEUS_ADDR')
         self.prom_client.start(prom_port, prom_addr)
+        self._export_ryu_config()
 
         # Start event notifier
         notifier_thread = self.notifier.start()
@@ -147,10 +150,6 @@ class Faucet(RyuAppBase):
             thread.name = name
             self.threads.append(thread)
 
-        # Register to API
-        self.api._register(self)
-        self.send_event_to_observers(EventFaucetExperimentalAPIRegistered())
-
     def _delete_deconfigured_dp(self, deleted_dpid):
         self.logger.info(
             'Deleting de-configured %s', dpid_log(deleted_dpid))
@@ -340,13 +339,3 @@ class Faucet(RyuAppBase):
             return
         if msg.reason == ryu_dp.ofproto.OFPRR_IDLE_TIMEOUT:
             self._send_flow_msgs(valve, valve.flow_timeout(time.time(), msg.table_id, msg.match))
-
-    def get_config(self):
-        """FAUCET experimental API: return config for all Valves."""
-        return get_config_for_api(self.valves_manager.valves)
-
-    def get_tables(self, dp_id):
-        """FAUCET experimental API: return config tables for one Valve."""
-        if dp_id in self.valves_manager.valves:
-            return self.valves_manager.valves[dp_id].dp.get_tables()
-        return {}
